'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/*******************************************************************************
express-mega-router - An Express middleware for dynamic routes and middleware.
Copyright (C) 2017 Ketchetwahmeegwun T. Southall / kts of kettek

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/
var pathToRegexp = require('path-to-regexp');
/**
 * An express middleware that allows the removing and adding of routes or other
 * middleware.
 *
 * @class MegaRouter
 * @constructor
 * @this {MegaRouter}
 * @property {Object[]} routes - Stored routes, such as: routes['GET'][0]
 * @param {String[]} [methods=['GET','HEAD','POST','PUT','DELETE','CONNECT','OPTIONS','TRACE','PATCH']]  - List of methods to add handlers for. This calls `addMethod` on each provided method. If unspecified, the methods outlined in RFC 7231 and RFC 5789 are used.
 * @example
 * const megaRouter = new require('express-megarouter');
 * app.use('*', megaRouter.getMiddleware());
 *
 * megaRouter.get('/', function(req, res, next) {
 *   next();
 * });
 *
 */

var MegaRouter = function () {
  function MegaRouter() {
    var _this = this;

    var methods = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : ['GET', 'HEAD', 'POST', 'PUT', 'DELETE', 'CONNECT', 'OPTIONS', 'TRACE', 'PATCH'];

    _classCallCheck(this, MegaRouter);

    methods.push('ALL');
    this.routes = {};
    methods.forEach(function (method) {
      _this.addMethod(method);
    });
  }
  /**
   * Adds handlers for a given method. Methods are expected to be in UPPER CASE. Lower-cased member functions will be added to the MegaRouter instance for 'method' and 'unmethod'.
   *
   * @param {string} method
   * @example
   * megaRouter.addMethod('CUSTOM');
   * megaRouter.custom('/', (req, res, next) => {
   *   res.send('My CUSTOM method response!');
   * });
   */


  _createClass(MegaRouter, [{
    key: 'addMethod',
    value: function addMethod(method) {
      var _this2 = this;

      this.routes[method] = [];
      // Add the method's lowercase name as a member function to this MegaRouter instance
      this[method.toLowerCase()] = function (route) {
        for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
          args[_key - 1] = arguments[_key];
        }

        // Allow for unlimited parameters
        if (args.length > 1) {
          for (var i = 0; i < args.length; i++) {
            _this2[method.toLowerCase()](route, args[i]);
          }
          return _this2;
        }
        // Allow for arrays
        if (args[0] instanceof Array) {
          for (var _i = 0; _i < args[0].length; _i++) {
            _this2[method.toLowerCase()](route, args[0][_i]);
          }
          return _this2;
        }
        // Callback type check
        if (!(args[0] instanceof Function)) {
          throw new Error('provided callback was not a Function');
        }
        // Alright, let's actually add the route
        if (!_this2.routes[method]) _this2.routes[method] = [];
        _this2.routes[method].push({ route: pathToRegexp(route), cb: args[0] });
        return _this2; // allow chaining
      };
      // Add the method's lowercase name with 'un' prepended to it as a member function to this MegaRouter instance
      this['un' + method.toLowerCase()] = function (route) {
        for (var _len2 = arguments.length, args = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
          args[_key2 - 1] = arguments[_key2];
        }

        // Bail early if the route doesn't exist
        if (!_this2.routes[method]) {
          return _this2;
        }
        // Check if it is a full route removal
        if (args.length == 0) {
          for (var i = _this2.routes[method].length - 1; i > 0; i--) {
            // Oh boy, this is just a little hackish
            if (_this2.routes[method][i].route.toString() == pathToRegexp(route).toString()) {
              _this2.routes[method].splice(i, 1);
            }
          }
          return _this2;
        }
        // Allow for removal by "unlimited" parameters
        if (args.length > 1) {
          for (var _i2 = 0; _i2 < args.length; _i2++) {
            _this2['un' + method.toLowerCase()](route, args[_i2]);
          }
          return _this2;
        }
        // Allow for removal by arrays
        if (args[0] instanceof Array) {
          for (var _i3 = 0; _i3 < args[0].length; _i3++) {
            _this2['un' + method.toLowerCase()](route, args[0][_i3]);
          }
          return _this2;
        }
        // Callback type check
        if (!(args[0] instanceof Function)) {
          throw new Error('provided callback was not a Function');
        }
        // Remove cb if it exists!
        for (var _i4 = 0; _i4 < _this2.routes[method].length; _i4++) {
          if (_this2.routes[method][_i4].cb === args[0]) {
            _this2.routes[method].splice(_i4, 1);
            return _this2;
          }
        }
        return _this2;
      };
    }
    /**
     * Returns the middleware function for this MegaRouter instance.
     *
     * @example
     * app.use('/*', megaRouter.getMiddleware());
     */

  }, {
    key: 'middleware',
    get: function get() {
      var _this3 = this;

      if (!this._middleware) {
        this._middleware = function (req, res, next) {
          var targets = [];
          var targetsIndex = 0;
          // Check the "ALL" route
          for (var i = 0; i < _this3.routes.ALL.length; i++) {
            if (_this3.routes.ALL[i].route.test(req.url)) {
              targets.push(_this3.routes.ALL[i].cb);
            }
          }
          // Check the method route
          if (_this3.routes[req.method]) {
            for (var _i5 = 0; _i5 < _this3.routes[req.method].length; _i5++) {
              if (_this3.routes[req.method][_i5].route.test(req.url)) {
                targets.push(_this3.routes[req.method][_i5].cb);
              }
            }
          }
          if (targets.length > 0) {
            var _step = function _step(index) {
              if (index < targets.length) {
                targets[index](req, res, function () {
                  _step(index + 1);
                });
              } else {
                next();
              }
            };

            _step(0);
          } else {
            next();
          }
        };
      }
      return this._middleware;
    }
  }]);

  return MegaRouter;
}();

module.exports = function (methods) {
  return new MegaRouter(methods);
};